<?php
declare(strict_types=1);

namespace Flat101\EnhancedEcommerce\Model\DataLayer\Generator;

use Flat101\EnhancedEcommerce\Model\DataLayer\DataProvider\AdapterDataProvider;
use Flat101\EnhancedEcommerce\Model\DataLayer\Dto;
use Flat101\EnhancedEcommerce\Model\Handler;
use Flat101\EnhancedEcommerce\Model\UnsetProperty;
use Flat101\EnhancedEcommerce\Model\Session;
use Flat101\EnhancedEcommerce\Helper\Config;
use Magento\Catalog\Api\CategoryRepositoryInterface;
use Magento\Catalog\Model\Product as ProductEntity;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Quote\Model\Quote\Item as QuoteItem;
use Magento\Sales\Model\Order\Item as OrderItem;
use Psr\Log\LoggerInterface;

class BehavioralEvent
{
    use UnsetProperty;

    /** @var Handler\ProductHandler */
    private $productHandler;

    /** @var Handler\ItemHandler */
    private $itemHandler;

    /** @var CategoryRepositoryInterface */
    private $categoryRepository;

    /** @var AdapterDataProvider */
    private $adapterDataProvider;

    /** @var Config */
    private $config;

    /** @var Session */
    private $session;

    /** @var LoggerInterface */
    private $logger;

    public function __construct(
        Handler\ProductHandler $productHandler,
        Handler\ItemHandler $itemHandler,
        CategoryRepositoryInterface $categoryRepository,
        AdapterDataProvider $adapterDataProvider,
        Config $config,
        Session $session,
        LoggerInterface $logger
    ) {
        $this->productHandler = $productHandler;
        $this->categoryRepository = $categoryRepository;
        $this->itemHandler = $itemHandler;
        $this->adapterDataProvider = $adapterDataProvider;
        $this->config = $config;
        $this->session = $session;
        $this->logger = $logger;
    }

    /**
     * @param $event
     * @param $eventCategory
     * @param $eventAction
     * @param $eventLabel
     * @return Dto\BehavioralEvent
     */
    public function generate($event, $eventCategory, $eventAction, $eventLabel = null): Dto\BehavioralEvent
    {
        $behavioralEvent = new Dto\BehavioralEvent();
        $behavioralEvent->event = $event;
        $behavioralEvent->eventCategory = $eventCategory;
        $behavioralEvent->eventAction = $eventAction;

        if (null !== $eventLabel) {
            $behavioralEvent->eventLabel = $eventLabel;
        }

        if (null === $eventLabel) {
            $this->unset($behavioralEvent, ['eventLabel']);
        }

        return $behavioralEvent;
    }
}
