<?php
declare(strict_types=1);

namespace Flat101\EnhancedEcommerce\Model\Handler\Click;

use Flat101\EnhancedEcommerce\Helper\Config;
use Flat101\EnhancedEcommerce\Model\DataLayer\Generator\ClickImpression as ClickImpressionUa;
use Flat101\EnhancedEcommerce\Model\DataLayerGa4\Generator\ClickImpression as ClickImpressionGa4;
use Flat101\EnhancedEcommerce\Model\Session;

class ClickCatalogImpressionHandler implements ClickImpressionHandlerInterface
{
    private const DEFAULT_LIST = 'Catalog Products';

    /** @var Config */
    private $config;

    /** @var Session */
    private $session;

    /** @var ClickImpressionUA */
    private $clickImpressionGenerator;
    
    /** @var ClickImpressionGa4 */
    private $clickImpressionGeneratorGa4;

    public function __construct(
        Session $session,
        ClickImpressionUa $clickImpressionGenerator,
        ClickImpressionGa4 $clickImpressionGeneratorGa4,
        Config $config
    ) {
        $this->session = $session;
        $this->clickImpressionGenerator = $clickImpressionGenerator;
        $this->clickImpressionGeneratorGa4 = $clickImpressionGeneratorGa4;
        $this->config = $config;
    }

    public function handle($product, $clickData = null): void
    {
        if (!is_null($clickData) && isset($clickData['GA4'])) {
            $this->session->setClickImpressionProducts(
                $this->clickImpressionGeneratorGa4->generate($product,
                    ($clickData['GA4']['list']) ?? self::DEFAULT_LIST,
                    $clickData['GA4']), false
            );
            if (!$this->config->getIsDisabledUaDatalayer() && isset($clickData['UA'])) {
                $this->session->setClickImpressionProducts(
                    $this->clickImpressionGenerator->generate(
                        $product,
                        ($clickData['UA']['list']) ?? self::DEFAULT_LIST,
                        $clickData['UA']), true
                );
            }
        }
    }
}
